#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <stdio.h>
#include <net/ethernet.h>
#include <sys/time.h>

#include "norm.h"
#include "norm_ip.h"
#include "norm_fddi.h"
#include "norm_misc.h"
#include "norm_debug.h"


int
norm_fddi_handler(NormPacket *p, Interface *nif)
{
  struct fddi_header *fh;
  struct snap_header *sh;
  u_char              buf1[FDDI_HDR_LEN];
  u_char              buf2[SNAP_LEN];

  D_ENTER;

  /*things may be unaligned - copy the header to ensure alignment*/
  memcpy(buf1, p->link_data, min(p->length, FDDI_HDR_LEN));
  fh = (struct fddi_header*)buf1;
  memcpy(buf2, p->link_data+FDDI_HDR_LEN, min(p->length-FDDI_HDR_LEN, SNAP_LEN));
  sh = (struct snap_header*)buf2;


  /*Is this an FDDI 802.2 or an FDDI SNAP frame?*/
  if ((sh->ssap==0xaa)&&(sh->dsap==0xaa))
    {
      /*It's a SNAP frame*/
      if (((sh->snap[0]) | (sh->snap[1]) | (sh->snap[2]) |
	   (sh->snap[4]==0)) &&(sh->snap[3]==8))
	{
	  /*It's a SNAP header containing an IP datagram*/
	  p->network_data = p->link_data + (FDDI_HDR_LEN+SNAP_LEN);
	  D_RETURN_(norm_ip_handler(p, nif));
	}
      else
	{
	  print_ether(NORM_LOGLEVEL, ("FDDI frame does not contain IP\n"));
	  D_RETURN_(-1);
	}
    }
  else
    {
      printf("XXX we don't handle FDDI 802.2 frames yet\n");
      D_RETURN_(-1);
    }
}


