%{

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <net/ethernet.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/time.h>
#include <errno.h>

#include "norm.h"
#include "norm_misc.h"
#include "norm_options.h"

#define YYERROR_VERBOSE 1

extern int yylex(void);
static int cfg_line = 1;
extern char* yytext;

static void
yyerror(char *msg)
{
        printf("Error at line %i in config file -- %s, will try to skip and continue.\n", cfg_line, msg);
}

%}

%union {
	int   i;
	char *s;
}

%token  INTERFACE READTRACE WRITETRACE TRUSTED IPADDR PASSIVE
%token  ETHER FDDI IP TCP UDP ICMP
%token  LAYER LINK NET TRANS
%token  NORM_IP_HEADER_LENGTH_CHECK NORM_IP_TOTAL_LENGTH_CHECK NORM_IP_HEADER_CHECKSUM_CHECK NORM_IP_HEADER_ADDR_CHECK NORM_IP_DROP_FRAGS_WITH_DF NORM_IP_CLEAR_DF NORM_IP_CHECK_FRAGMENT_OFF NORM_IP_HEADER_CLEAR_TOS NORM_IP_CHECK_OPTIONS NORM_IP_SET_TTL NORM_IP_CLEAR_RF
%token  NORM_UDP_VERIFY_CKSUM NORM_UDP_VERIFY_LENGTH
%token  NORM_TCP_NEED_SYN NORM_TCP_VERIFY_CKSUM NORM_TCP_CHECK_FLAGS NORM_TCP_CHECK_LENGTHS NORM_TCP_CHECK_OPTIONS NORM_TCP_CLEAR_RESERVED
%token  NORM_ICMP_ZERO_CODE_CHECK NORM_ICMP_DROP_BROADCAST NORM_ICMP_DROP_MULTICAST NORM_ICMP_VERIFY_CKSUM NORM_ICMP_DROP_SOURCEQUENCH
%token  YES NO STRING NUM IP DEBUGLEVEL

%type   <i> options option trustdef nifdef leveldef linklist linkprot netlist netprot transplist transplist normswitch switch
%type   <s> STRING IPADDR
%type   <i> INTERFACE TRUSTED PASSIVE
%type   <i> ETHER FDDI IP TCP UDP ICMP
%type   <i> LAYER LINK NET TRANS
%type   <i> YES NO NUM DEBUGLEVEL
%type   <i> NORM_IP_HEADER_LENGTH_CHECK NORM_IP_TOTAL_LENGTH_CHECK NORM_IP_HEADER_CHECKSUM_CHECK NORM_IP_HEADER_ADDR_CHECK NORM_IP_DROP_FRAGS_WITH_DF NORM_IP_CLEAR_DF NORM_IP_CHECK_FRAGMENT_OFF NORM_IP_HEADER_CLEAR_TOS NORM_IP_CHECK_OPTIONS NORM_IP_SET_TTL NORM_IP_CLEAR_RF
%type   <i> NORM_UDP_VERIFY_CKSUM NORM_UDP_VERIFY_LENGTH
%type   <i> NORM_TCP_NEED_SYN NORM_TCP_VERIFY_CKSUM NORM_TCP_CHECK_FLAGS NORM_TCP_CHECK_LENGTHS NORM_TCP_CHECK_OPTIONS NORM_TCP_CLEAR_RESERVED
%type   <i> NORM_ICMP_ZERO_CODE_CHECK NORM_ICMP_DROP_BROADCAST NORM_ICMP_DROP_MULTICAST NORM_ICMP_VERIFY_CKSUM NORM_ICMP_DROP_SOURCEQUENCH

%%
configfile: options                     {  }
           | null
           ;
null:	   /* null */   	        {  }
           ;
options:     option                     {  }
           | options option             {  }
           ;
option:      '\n'                       { cfg_line++; }
           | trustdef                   {  }
           | nifdef                     {  }
           | leveldef                   {  }
           | normswitch                 {  }
           | error '\n'                 {  }
           ;
trustdef:    TRUSTED IPADDR '/' NUM     { norm_misc_add_trusted_network($2, $4); }
           ;
nifdef:      READTRACE STRING           { if (options.in_tracefile) free(options.in_tracefile); options.in_tracefile = $2; }
           | WRITETRACE STRING          { if (options.out_tracefile) free(options.out_tracefile); options.out_tracefile = $2; }
           | INTERFACE niflist          {  }
           ;
niflist:     null                       {  }
           | STRING niflist             { norm_misc_add_network_interface($1); }
           ;
leveldef:    LAYER LINK  linklist       {  }
           | LAYER NET   netlist        {  }
           | LAYER TRANS transplist     {  }
           ;
linklist:    null                       { }
           | linkprot linklist          { }
           ;
linkprot:    ETHER			{ options.show_ether = TRUE; }
           | FDDI			{ options.show_fddi  = TRUE; }
           ;
netlist:     null                       { }
           | netprot netlist            { }
           ;
netprot:     IP				{ options.show_ip    = TRUE; }
           ;
transplist:  null			{ }
           | transpprot transplist      { }
           ;
transpprot:  TCP			{ options.show_tcp   = TRUE; }
           | UDP			{ options.show_udp   = TRUE; }
           | ICMP			{ options.show_icmp  = TRUE; }
           ;
normswitch:  PASSIVE                       switch { options.passive                       = $2; }
           | NORM_IP_DROP_FRAGS_WITH_DF    switch { options.ip_drop_fragments_with_df     = $2; }
           | NORM_IP_CLEAR_DF              switch { options.ip_clear_df_and_offset        = $2; }
           | NORM_IP_CHECK_FRAGMENT_OFF    switch { options.ip_check_fragment_offset      = $2; }
           | NORM_IP_HEADER_LENGTH_CHECK   switch { options.ip_header_length_check        = $2; }
           | NORM_IP_TOTAL_LENGTH_CHECK    switch { options.ip_total_length_check         = $2; }
           | NORM_IP_HEADER_CHECKSUM_CHECK switch { options.ip_header_checksum_check      = $2; }
           | NORM_IP_HEADER_ADDR_CHECK     switch { options.ip_header_addr_check          = $2; }
           | NORM_IP_HEADER_CLEAR_TOS      switch { options.ip_header_clear_tos           = $2; }
           | NORM_IP_CHECK_OPTIONS         switch { options.ip_check_options              = $2; }
           | NORM_IP_SET_TTL               YES NUM{ options.ip_check_ttl = 1; options.ip_ttl_value = $3; }
           | NORM_IP_SET_TTL               NO     { options.ip_check_ttl                  = 0; }
           | NORM_IP_CLEAR_RF              switch { options.ip_clear_rf                   = $2; }
           | NORM_ICMP_ZERO_CODE_CHECK     switch { options.icmp_drop_nonzero_code        = $2; }
	   | NORM_ICMP_DROP_BROADCAST      switch { options.icmp_drop_broadcast           = $2; }
	   | NORM_ICMP_DROP_MULTICAST      switch { options.icmp_drop_multicast           = $2; }
           | NORM_ICMP_VERIFY_CKSUM        switch { options.icmp_verify_checksum          = $2; }
           | NORM_ICMP_DROP_SOURCEQUENCH   switch { options.icmp_drop_sourcequench        = $2; }
           | NORM_UDP_VERIFY_CKSUM         switch { options.udp_verify_checksum           = $2; }
           | NORM_UDP_VERIFY_LENGTH        switch { options.udp_verify_length             = $2; }
           | NORM_TCP_NEED_SYN             switch { options.tcp_need_syn                  = $2; }
           | NORM_TCP_VERIFY_CKSUM         switch { options.tcp_verify_checksum           = $2; }
           | NORM_TCP_CHECK_FLAGS          switch { options.tcp_check_flags               = $2; }
           | NORM_TCP_CHECK_LENGTHS        switch { options.tcp_check_lengths             = $2; }
           | NORM_TCP_CHECK_OPTIONS        switch { options.tcp_check_options             = $2; }
           | NORM_TCP_CLEAR_RESERVED       switch { options.tcp_clear_reserved            = $2; }
	   | DEBUGLEVEL                    NUM    { options.debuglevel                    = $2; }
           ;
switch:      YES			{ $$ = TRUE;  }
           | NO			        { $$ = FALSE; }
           ;
%%
